### MAPPING

La configuration se fait en YAML dans le fichier `config/migration_mapping.yaml`

Exemple : 

Voir `/tests/Application/config/packages/dedi_sylius_migration.yaml` pour un exemple d'import de données catalogue depuis une base de données Legacy ShopByDedi vers Sylius

Dans le cas des clés étrangères l'ordre d'import a son importance. L'import se fait dans l'ordre de définition des tables dans le fichier YAML.

La structure de la configuration se fait tel que suit : 

```yaml
migration_mapping:
  [table_destination]:
    table: [table_source]
    sql: [sql_request]
    primary_key: [table_source.colonne_primary_key]
    order:
      column: [table_source.colonne]
      sort: ['ASC' ou 'DESC']
    identifier_mapping:
      - { source_table: [table_source], source_column: [table_source.colonne], dest_column: [table_destination.colonne] }
      - ...
    filter:
      - { filter_column: [table_source.colonne], value: [valeur], type: [comparison|like|is_null] }
      - ...
    search:
      - { source_table: [table_source], value_column: [table_source.colonne], type: [comparison|like|is_null] }
      - ...
    fields:
      [table_destination.colonne]: [table_source.colonne]
      [table_destination.colonne]:
        field: [table_source.colonne] 
        sql: [sql_request]
      [table_destination.colonne]:
        alias: [alias]
        expr: [clause_sql]
      [table_destination.colonne]:
        field: [table_source.colonne]
        transformer: [transformer_name]
      [table_destination.colonne]:
        field: [table_source.colonne]
        identifier_mapping:
          source_table: [identifier_mapping.source_table]
          source_column: [identifier_mapping.source_column]
    logging_columns:
      - [table_source.colonne]
      - ...
    after_migrate:
      sql:
        - [sql_request]
        - ...
    handler: [handler_name]
    error_handler:
        [error_handler_name]: ~
        [error_handler_name]:
            [config_key]: [config_value]
            ...
    group: [group_name]
```

###migration_mapping > table_destination
**type**: `string`

Table de la base de données cible dans laquelle insérer les données
```yaml
migration_mapping:
  sylius_product:
```

###migration_mapping > table_destination.table
**type**: `string`

[Optionnel] Table de la base de données source depuis laquelle récupérer les données (executera un SELECT * FROM [table])

Cette option est surchargée par l'option `table_destination.sql`

```yaml
migration_mapping:
  sylius_product:
    table: produit
```

###migration_mapping > table_destination.sql
**type**: `string`

[Optionnel] Requête SQL de récupération de données executée sur la base de donnée source (surcharge l'option table). Avec ce mode, les `fields` de type `expr` seront ignorés, ainsi que les options `filter` et `order`

Cette option surcharge l'option `table_destination.table`

```yaml
migration_mapping:
  sylius_product:
    sql: 'SELECT * FROM produit p LEFT JOIN produit_nom pn ON pn.produit_id = p.produit_id'
```

###migration_mapping > table_destination.primary_key
**type**: `string`

Clé primaire de la table source utilisée dans les logs pour identifier les lignes et si une "legacy column" est spécifiée

```yaml
migration_mapping:
  sylius_product:
    primary_key: produit_id
```

###migration_mapping > table_destination.order
**type**: `array associatif`

Définition de l'ordre à donner aux données récupérées dans la base de données source. C'est utile si vous savez qu'il y a des doublons : l'ordre permettra de récupérer en premier les données les plus à jour, qui seront écrites en base, les données les moins à jour crééront une erreur (duplicate key)

Cette option est ignorée si l'option `table_destination.sql` est spécifiée

`column` (**défaut**: `table_destination.primary_key`) : nom de la ou les colonne(s) sur la(es)quelle(s) ordonner les résultats

`sort`(**défaut**: `DESC`) : type de classement attendu `ASC` ou `DESC`

```yaml
migration_mapping:
  sylius_product:
    order:
      column: produit_date_ajout
      sort: 'ASC'
```
###migration_mapping > table_destination.filter
**type**: `array liste`

Définition des clauses (une clause par ligne) de filtrage sur la requête executée sur la base de données source. 

Cette option va ajouter un `WHERE` dans la requête, avec les clauses aggrégées par un `AND`

Cette option est ignorée si l'option `table_destination.sql` est spécifiée

`search_column` : nom de la colonne sur laquelle filter

`value` : valeur recherche dans la colonne de filtrage

`type` : type de comparaison parmis `comparison` (`= value`), `not_comparison` (`!= value`) `like` (`LIKE %value%`), `is_null` (`IS NULL`, `value` est ignoré)

```yaml
migration_mapping:
  sylius_product:
    filter:
      - { search_column: produit_statut_id, value: 1, type: comparison }
      - { search_column: produit_ref, type: is_null }
      ...
```

###migration_mapping > table_destination.search
**type**: `array liste`

Définition des clauses (une clause par ligne) de recherche de données dans la base de données cible avant insertion. 

Cette option va générer un `SELECT * FROM [table_destination] WHERE [search_column] [type] [value_column]` executé avant d'insérer une ligne. 

Si la requête retourne un résultat, c'est un UPDATE qui sera exécuté avec la même clause `WHERE` 

Cette option est ignorée si l'option `table_destination.sql` est spécifiée

`search_column` : nom de la colonne sur laquelle faire la recherche

`value_column` : nom de la colonne de la ligne de données à insérer à utiliser pour la recherche

`type` : type de comparaison parmi `comparison` (`= value`), `not_comparison` (`!= value`) `like` (`LIKE %value%`), `is_null` (`IS NULL`, `value` est ignoré)

```yaml
migration_mapping:
  sylius_product:
    search:
      - { search_column: code, value: produit_ref, type: comparison }
      ...
```

###migration_mapping > table_destination.group
**type**: `string`

Clé permettant de regrouper des tables à migrer en groupe. Par exemple : php bin/console dedi:migration:execute --group=catalogue

```yaml
migration_mapping:
  sylius_product:
      group: catalogue
```

###migration_mapping > table_destination.fields
**type**: `array associatif`

Liste des colonnes à récupérer dans la base de données source et insérer ou mettre à jour dans la base de données cible. 

Il est nécessaire que chaque clé de ce tableau existe dans la ligne récupérée en base de données. Il faut donc que la colonne eixste dans la table (ou dans la requête executée si l'option `table_destination.sql` est spécifiée)

Chaque ligne du tableau correspond à un couple donnée source / donnée cible

**Mapping simple**

```yaml
table_destination.colonne: table_source.colonne
```

La donnée `table_source.colonne` sera insérée dans la colonne `table_destination.colonne`

```yaml
migration_mapping:
  sylius_product:
    fields:
      code: produit_ref
```

**Résultat d'une requête**

```yaml
table_destination.colonne:
    field: table_source.colonne
    sql: sql_request
```

La requête `sql_request` sera executée sur la base de données cible (avec passage de `field` en paramètre de la requête). Son résultat sera inséré dans `table_destination.colonne`

C'est par exemple utile pour récupérer le nouvel ID d'une ligne précédemment insérée à partir de son ID dans l'ancienne base

```yaml
migration_mapping:
  sylius_product:
    fields:
      tax_id:
        field: taxe_id
        sql: 'SELECT id FROM tax WHERE legacy_id = :taxe_id'
```

**Résultat d'une expression SQL**

```yaml
table_destination.colonne:
    alias: alias
    expr: clause_sql
```

La `clause_sql` sera ajoutée au `SELECT` sous l'alias `alias` et son résultat sera inséré `table_destination.colonne` (`SELECT [clause_sql] AS [alias]`)

Cette option est ignorée si l'option `table_destination.sql` est spécifiée

```yaml
migration_mapping:
  sylius_product:
    fields:
      visible:
        alias: visible
        expr: 'IF (produit_statut_id = 1, 1, 0)'
```

**Appliquer un Transformer sur un résultat**

Il est possible, pour tout les types de mapping de champs précédemment présentés, d'appliquer un Transformer : il s'agit d'un service qui reçoit les données issues de la base de données source en entrée et les retournes après execution d'un traitement PHP.

Les Transformers permettent d'appliquer des transformations qui ne sont pas possible avec les simples clauses SQL (ex : slugifier un nom)

Voir la [documentation sur les Transformers](TRANSFORMER.md) pour plus de détails sur ces services

```yaml
table_destination.colonne:
    field: table_source.colonne
    transformer: transformer_name
```

La valeur `transformer_name` doit correspondre à un `Type` retourné par l'un des Transformers déclarés dans le projet

```yaml
migration_mapping:
  sylius_product:
    fields:
      visible:
        field: produit_nom_nom
        transformer: slugify 
```

**Trouver une valeur depuis le mapping d'identifiants**

Il est possible, pour tout les types de mapping de champs précédemment présentés, de trouver sa valeur depuis des mappings `identifier_mapping` renseignés précédemment dans la configuration.


```yaml
migration_mapping:
  sylius_taxon:
    primary_key: categorie_id
    identifier_mapping:
      - { source_table: categorie, source_column: categorie_id, dest_column: id }

  sylius_taxon_translation:
    fields:
      translatable_id:
        field: categorie_id
        identifier_mapping:
          source_table: categorie
          source_column: categorie_id
```

Dans cet exemple nous configurons un mapping d'identifiant `identifier_mapping` pour la table de destination `sylius_taxon`.

La valeur de la colonne `categorie_id` de la table source `categorie` sera donc mappée sur la colonne `id` de la table de destination `sylius_taxon`.

La valeur de la colonne `translatable_id` de `sylius_taxon_translation` étant liée à la colonne source `categorie_id`, elle pourra alors être retrouvée automatiquement en indiquant dans quel `identifier_mapping.source_table` et `identifier_mapping.source_column` la valeur source et donc de destination peuvent être trouvée.


###migration_mapping > table_destination.logging_columns
**type**: `array liste`

Défini les colonnes à inclure dans le fichier de log CSV généré par l'import. Cela permet d'ajouter des informations permettant de mieux comprendre ou regrouper les messages d'erreur. 

```yaml
table_destination.colonne:
    logging_columns: 
      - table_source.colonne
      ...
```

Par exemple, si un produit avec une référence spécifique ne passe pas, la traduction du même produit ne passera pas. Ajouter la référence du produit à la ligne d'erreur remontée par l'import de la traduction permettra d'identifier rapidement le lien "produit ne passe pas, traduction ne passe pas"

```yaml
migration_mapping:
  sylius_product_translation:
    logging_columns:
      - produit_ref
      - produit_ref_fournisseur
      ...
```

###migration_mapping > table_destination.identifier_mapping
**type**: `array liste`

Défini les colonnes qui seront stockées pour mapper les identifiants dans `config.tmp_mapping_table_name`.
Ces mappings seront utilisées par `table_destination.search` afin de retrouver les enregistrements existant dans la base de donnée de destination.

```yaml
migration_mapping:
  sylius_product_translation:
    identifier_mapping:
      - { source_table: produit_nom, source_column: produit_nom_id, dest_column: id }
    search:
      - { source_table: produit_nom, value_column: produit_nom_id, type: comparison }
```

Dans cet exemple, après insertion de la ligne de la base de donnée de destination, la valeur de la colonne `produit_nom_id` de la table source `produit_nom`, et l'identifiant de la ligne enregistrée correspondant à la colonne `id` de la table de destination `sylius_product_translation` seront enregistrée dans la table `config.tmp_mapping_table_name`.

Lors d'un ré-import des données, `table_destination.search` ira chercher dans `config.tmp_mapping_table_name` la valeur de la ligne dont `source_table_name` vaut `search.source_table`, `source_column_name` vaut `search.value_column` et `source_id` est égal à la valeur de `search.value_column` dans la ligne en cours.

Si quelque chose est trouvé, alors la valeur trouvée `dest_id` de `identifier_mapping.dest_column` est utilisé comme filtre dans la requete afin de mettre à jour la ligne existante dans `table_destination`.

**Appliquer un Handler sur une ligne**

Il est possible d'appeler un handler auquel seront passées chaque ligne de données récupérées dans la base de données source. 

Dans ce cas, seul le handler sera appelé, le reste de la configuration (mapping field par exemple) est ignoré. 

Voir la [documentation sur les Handlers](HANDLER.md) pour plus de détails sur ces services

```yaml
table_destination:
    handler: handler_name
```

La valeur `handler_name` doit correspondre à un `Type` retourné par l'un des Handlers déclarés dans le projet

```yaml
migration_mapping:
  sylius_taxon:
      transformer: categorie_to_taxon_handler
```

**Utiliser un error handler**

Il est possible d'appeler un handler appelé lorsqu'une erreur survient lors de l'import d'une ligne de données. 

Dans ce cas, la fonction `can` du ErrorHandler est appelée pour vérifier si l'exception doit être traitée par le handler : dans cette fonction, on pourra vérifier le type d'Exception ou le message pour décider si le handler doit être appliqué ou non.

Le fonction `getErrorBubbling` détermine si l'erreur doit être relancée après application du handler. Cela permet de décider si on laisse ensuite l'execution classique de la gestion d'erreur : catch de l'erreur et log. 

Voir la [documentation sur les ErrorHandlers](ERROR_HANDLER.md) pour plus de détails sur ces services

```yaml
table_destination:
    error_handler_name: configuration
```

La valeur `error_handler_name` doit correspondre à un `Type` retourné par l'un des Error Handlers déclarés dans le projet

La valeur `configuration` est passée au handler et accessible via la fonction `getConfiguration`. Elle permet de passer une configuration utilisée lors de l'execution du handler. Cette valeur doit être soit un array clé / valeur, soit ~.

```yaml
migration_mapping:
  sylius_taxon:
      send_email_error: ~
      
```

```yaml
migration_mapping:
  sylius_taxon:
      save_duplicate:
          columns: [ code ]
      
```
