<?php

declare(strict_types=1);

namespace Dedi\SyliusMigrationPlugin\DependencyInjection;

use Symfony\Component\Config\Definition\Builder\TreeBuilder;
use Symfony\Component\Config\Definition\ConfigurationInterface;

final class Configuration implements ConfigurationInterface
{
    public function getConfigTreeBuilder(): TreeBuilder
    {
        $treeBuilder = new TreeBuilder('dedi_sylius_migration');
        $treeBuilder
            ->getRootNode()
            ->children()
                ->arrayNode('config')
                    ->children()
                        ->booleanNode('commit')
                            ->defaultTrue()
                        ->end()
                        ->integerNode('debug_limit_statements')
                            ->defaultValue(-1)
                        ->end()
                        ->booleanNode('generate_relative_path_for_images')
                            ->defaultTrue()
                        ->end()
                        ->scalarNode('legacy_site_url')
                            ->isRequired()
                        ->end()
                        ->scalarNode('legacy_site_media_path')
                            ->isRequired()
                        ->end()
                        ->scalarNode('new_site_media_path')
                            ->defaultValue('legacy')
                        ->end()
                        ->scalarNode('new_site_url')
                            ->defaultValue('')
                        ->end()
                        ->scalarNode('media_download_method')
                            ->defaultValue('on_migrate')
                        ->end()
                        ->scalarNode('tmp_mapping_table_name')
                            ->defaultValue('tmp_dedi_migration_mapping')
                        ->end()
                    ->end()
                ->end()
            ->end()
        ;

        $treeBuilder = $this->getMigrationMappingConfig($treeBuilder);
        $treeBuilder = $this->getMigrationActionConfig($treeBuilder);

        return $treeBuilder;
    }

    private function getMigrationMappingConfig(TreeBuilder $treeBuilder): TreeBuilder
    {
        $rootNode = $treeBuilder
            ->getRootNode()
        ;

        $rootNode
            ->children()
                ->arrayNode('migration_mapping')
                ->useAttributeAsKey('name')
                ->arrayPrototype()
                    ->children()
                        ->scalarNode('target_table')->end()
                        ->scalarNode('group')->end()
                        ->scalarNode('table')->end()
                        ->scalarNode('sql')->end()
                        ->scalarNode('primary_key')->end()
                        ->arrayNode('order')
                            ->children()
                                ->scalarNode('column')->end()
                                ->scalarNode('sort')->end()
                            ->end()
                        ->end()
                        ->arrayNode('limit')
                            ->addDefaultsIfNotSet()
                            ->children()
                                ->scalarNode('offset')->defaultValue(0)->end()
                                ->scalarNode('step')->defaultValue(100)->end()
                                ->scalarNode('limit')->defaultValue(-1)->end()
                            ->end()
                        ->end()
                        ->arrayNode('filter')
                            ->arrayPrototype()
                                ->children()
                                    ->scalarNode('filter_column')->end()
                                    ->scalarNode('value')->end()
                                    ->scalarNode('type')->end()
                                ->end()
                            ->end()
                        ->end()
                        ->arrayNode('fields')
                            ->useAttributeAsKey('name')
                            ->arrayPrototype()
                                ->beforeNormalization()
                                    ->ifString()->then(function ($v) { return ['field' => $v]; })
                                ->end()
                                ->children()
                                    ->scalarNode('field')->defaultNull()->end()
                                    ->scalarNode('sql')->defaultNull()->end()
                                    ->scalarNode('value')->defaultNull()->end()
                                    ->scalarNode('alias')->defaultNull()->end()
                                    ->scalarNode('expr')->defaultNull()->end()
                                    ->scalarNode('transformer')->defaultNull()->end()
                                    ->arrayNode('identifier_mapping')
                                        ->children()
                                            ->scalarNode('source_table')->defaultNull()->end()
                                            ->scalarNode('source_column')->defaultNull()->end()
                                            ->scalarNode('dest_table')->defaultNull()->end()
                                        ->end()
                                    ->end()
                                ->end()
                            ->end()
                        ->end()
                        ->scalarNode('handler')->defaultNull()->end()
                        ->arrayNode('error_handlers')
                            ->useAttributeAsKey('name')
                            ->arrayPrototype()
                                ->beforeNormalization()
                                    ->always(function ($v) { return ['configuration' => $v]; })
                                ->end()
                                ->children()
                                    ->variableNode('configuration')
                                    ->end()
                                ->end()
                            ->end()
                        ->end()
                        ->arrayNode('search')
                            ->arrayPrototype()
                                ->children()
                                    ->scalarNode('source_table')->end()
                                    ->scalarNode('value_column')->end()
                                    ->scalarNode('type')->end()
                                ->end()
                            ->end()
                        ->end()
                        ->arrayNode('identifier_mapping')
                            ->arrayPrototype()
                                ->children()
                                    ->scalarNode('source_table')->end()
                                    ->scalarNode('dest_table')->end()
                                    ->scalarNode('source_column')->end()
                                    ->scalarNode('dest_column')->end()
                                ->end()
                            ->end()
                        ->end()
                        ->arrayNode('logging_columns')
                            ->scalarPrototype()->end()
                        ->end()
                        ->arrayNode('date_fields')
                            ->scalarPrototype()->end()
                        ->end()
                        ->arrayNode('after_migrate')
                            ->children()
                                ->arrayNode('sql')
                                    ->scalarPrototype()->end()
                                ->end()
                            ->end()
                        ->end()
                    ->end()
                ->end()
            ->end()
        ;

        return $treeBuilder;
    }

    private function getMigrationActionConfig(TreeBuilder $treeBuilder): TreeBuilder
    {
        $rootNode = $treeBuilder
            ->getRootNode()
        ;

        $rootNode
            ->children()
            ->arrayNode('migration_actions')
                ->children()
                    ->arrayNode('truncate_table_before_migration')
                        ->scalarPrototype()->end()
                    ->end()
                    ->arrayNode('execute_query_before_migration')
                        ->scalarPrototype()->end()
                    ->end()
                    ->arrayNode('pre_migration_executors')
                        ->beforeNormalization()->castToArray()->end()
                        ->scalarPrototype()->end()
                    ->end()
                    ->arrayNode('create_tmp_mapping_table_before_migration')
                        ->beforeNormalization()->castToArray()->end()
                        ->scalarPrototype()->end()
                    ->end()
                    ->arrayNode('remove_tmp_mapping_table_after_migration')
                        ->beforeNormalization()->castToArray()->end()
                        ->scalarPrototype()->end()
                    ->end()
                ->end()
            ->end()
        ;

        return $treeBuilder;
    }
}
